@@ -17,6 +17,8 @@ is actually data returned from rest API response.
 
 CommandResult.discovered hold all rest endpoint.
 
+https://www.dell.com/support/manuals/en-us/idrac9-lifecycle-controller-v4.x-series/idrac9_4.00.00.00_redfishapiguide_pub/redfish-resources?guid=guid-d3e85da8-5d22-4eb1-82ff-d2fdd4cd7730&lang=en-us
+
 Author Mus spyroot@gmail.com
 """
 import argparse
@@ -32,10 +34,15 @@ from abc import abstractmethod
 from typing import Optional, Tuple, Dict, Any
 import re
 import logging
+from datetime import datetime
+from functools import cache
+from functools import cached_property
 
-from idrac_ctl.shared import ApiRequestType, RedfishAction, ScheduleJobType
-from idrac_ctl.cmd_utils import save_if_needed
+from idrac_ctl.custom_argparser.customer_argdefault import CustomArgumentDefaultsHelpFormatter
+from idrac_ctl.shared import ApiRequestType, RedfishAction, ScheduleJobType, IDRAC_JSON, IDRAC_API, JobApplyTypes
+from .cmd_utils import save_if_needed
 from .cmd_exceptions import AuthenticationFailed
+from .cmd_exceptions import MissingMandatoryArguments
 from .cmd_exceptions import PostRequestFailed
 from .cmd_exceptions import MissingResource
 from .cmd_exceptions import UnexpectedResponse
@@ -43,10 +50,12 @@ from .cmd_exceptions import ResourceNotFound
 from .cmd_exceptions import PatchRequestFailed
 from .cmd_exceptions import DeleteRequestFailed
 from .cmd_exceptions import UnsupportedAction
+from .cmd_exceptions import InvalidArgumentFormat
+from .shared import JobState
 
 """Each command encapsulate result in named tuple"""
 CommandResult = collections.namedtuple("cmd_result",
-                                       ("data", "discovered", "extra"))
+                                       ("data", "discovered", "extra", "error"))
 
 module_logger = logging.getLogger('idrac_ctl.idrac_manager')
 
@@ -79,12 +88,17 @@ class IDracManager:
         self._is_verify_cert = insecure
         self._x_auth = x_auth
         self._is_debug = is_debug
-
+        self._default_method = "https://"
         self.logger = logging.getLogger(__name__)
 
         self.content_type = {'Content-Type': 'application/json; charset=utf-8'}
         self.json_content_type = {'Content-Type': 'application/json; charset=utf-8'}
 
+        self._manage_servers_obs = []
+        self._manage_chassis_obs = []
+        # mainly to track query sent , for unit test
+        self.query_counter = 0
+
         # run time
         self.action_targets = None
         self.api_endpoints = None
@@ -93,19 +107,19 @@ class IDracManager:
         self._delete_success_responses = [200, 201, 202, 203, 204]
 
     @property
-    def idrac_ip(self):
+    def idrac_ip(self) -> str:
         return self._idrac_ip
 
     @property
-    def username(self):
+    def username(self) -> str:
         return self._username
 
     @property
-    def password(self):
+    def password(self) -> str:
         return self._password
 
     @property
-    def x_auth(self):
+    def x_auth(self) -> str:
         return self._x_auth
 
     def __init_subclass__(cls, scm_type=None, name=None, **kwargs):
@@ -203,9 +217,11 @@ class IDracManager:
 
         if self.x_auth is not None:
             return loop.run_in_executor(
-                None, functools.partial(requests.get, r,
-                                        verify=self._is_verify_cert,
-                                        headers=headers)
+                None, functools.partial(
+                    requests.get, r,
+                    verify=self._is_verify_cert,
+                    headers=headers
+                )
             )
         else:
             return loop.run_in_executor(
@@ -279,11 +295,19 @@ class IDracManager:
 
     def sync_invoke(self, api_call: ApiRequestType, name: str, **kwargs) -> CommandResult:
         """Synchronous invocation of target command
-        :param api_call: A enum for command.
-        :param name: A name for command to differentiate sub-commands.
-        :param kwargs: Args passed to a command.
-        :return: Return result depends on actual command.
+        :param name: a name for command to differentiate sub-commands
+        :param api_call: enum i.e. a type command that we need invoke
+        :param kwargs: arguments passed to a command
+        :return: Return result depends on actual command,
+                 encapsulated in generic CommandResult
         """
+        if len(self._username) == 0:
+            raise ValueError("Username is empty string.")
+        if len(self._password) == 0:
+            raise ValueError("Password is empty string.")
+        if len(self._idrac_ip) == 0:
+            raise ValueError("IDRAC IP is empty string.")
+
         kwargs.update(
             {
                 "idrac_ip": self._idrac_ip,
@@ -293,36 +317,68 @@ class IDracManager:
         )
         return self.invoke(api_call, name, **kwargs)
 
+    def get_job(self,
+                job_id: str,
+                data_type: Optional[str] = "json",
+                do_async: Optional[bool] = False) -> dict:
+        """Query information for particular job.
+        :param job_id: iDRAC job_id JID_744718373591
+        :param do_async: note async will subscribe to an event loop.
+        :param data_type: json or xml
+        :return: CommandResult and if filename provide will save to a file.
+        """
+        headers = {}
+        if data_type == "json":
+            headers.update(self.json_content_type)
+
+        r = f"{self.idrac_members}/Oem/Dell/Jobs/{job_id}"
+        return self.base_query(r, do_expanded=True, do_async=do_async).data
+
     def fetch_job(self,
                   job_id: str,
                   sleep_time: Optional[int] = 2,
-                  wait_for: Optional[int] = 200):
+                  wait_for: Optional[int] = 200,
+                  wait_status: Optional[bool] = True,
+                  wait_completion: Optional[bool] = True):
         """synchronous fetch a job from iDRAC and wait for completion.
         :param wait_for:  by default, we wait status code 200 based on spec.
         :param job_id: job id as it returned from a task by idrac
         :param sleep_time: sleep and wait.
+        :param wait_status: wait for http status.
+        :param wait_completion: wait for completion a task.
         :return: Nothing
         :raise AuthenticationFailed MissingResource
         """
         last_update = 0
         percent_done = 0
+
+        # job might be already done.
+        jb = self.get_job(job_id)
+        if IDRAC_JSON.JobState in jb:
+            current_state = jb[IDRAC_JSON.JobState]
+            if current_state == JobState.Completed.value:
+                return self.api_success_msg(True, message=f"Job {job_id} completed.")
+
         with tqdm(total=100) as pbar:
             while True:
-                resp = self.api_get_call(f"https://{self.idrac_ip}/redfish/v1/"
-                                         f"TaskService/Tasks/{job_id}", hdr={})
+                resp = self.api_get_call(f"{self._default_method}{self.idrac_ip}"
+                                         f"{IDRAC_API.IDRAC_TASKS}{job_id}", hdr={})
                 if resp.status_code == 401:
                     AuthenticationFailed("Authentication failed.")
                 elif resp.status_code == 404:
-                    raise MissingResource("Missing resource.")
-                elif resp.status_code == wait_for:
+                    raise MissingResource(f"Task {job_id} not found.")
+
+                elif resp.status_code == wait_for and wait_status:
                     resp_data = resp.json()
                     return resp_data
                 elif resp.status_code == 202:
                     resp_data = resp.json()
-                    if 'TaskStatus' in resp_data and resp_data['TaskStatus'] == 'OK':
-                        if 'PercentComplete' in resp_data:
+                    if IDRAC_JSON.TaskStatus in resp_data and resp_data[IDRAC_JSON.TaskStatus] == 'OK':
+                        if IDRAC_JSON.JobState in resp_data:
+                            current_state = resp_data[IDRAC_JSON.JobState]
+                        if IDRAC_JSON.PercentComplete in resp_data:
                             try:
-                                percent_done = int(resp_data['PercentComplete'])
+                                percent_done = int(resp_data[IDRAC_JSON.PercentComplete])
                             except TypeError:
                                 pass
                             if percent_done > last_update:
@@ -330,7 +386,11 @@ class IDracManager:
                                 inc = percent_done - pbar.n
                                 pbar.update(n=inc)
                             time.sleep(sleep_time)
+                elif wait_completion and current_state == JobState.Completed.value:
+                    return resp_data
                 else:
+                    if IDRAC_JSON.JobState in resp_data:
+                        current_state = resp_data[IDRAC_JSON.JobState]
                     self.logger.error("unexpected status code", resp.status_code)
                     time.sleep(sleep_time)
 
@@ -365,7 +425,7 @@ class IDracManager:
         if response.status_code == 401:
             raise AuthenticationFailed("Authentication failed.")
         if response.status_code == 404:
-            error_msg = IDracManager.parse_error(response)
+            error_msg, json_error = IDracManager.parse_error(response)
             raise ResourceNotFound(error_msg)
         if response.status_code != 200:
             raise UnexpectedResponse(f"Failed acquire result. Status code {response.status_code}")
@@ -376,15 +436,16 @@ class IDracManager:
         """
         headers = {}
         headers.update(self.json_content_type)
-        r = f"https://{self._idrac_ip}/redfish/v1/Dell/Managers" \
-            f"/iDRAC.Embedded.1/DellLCService"
+        r = f"{self._default_method}{self._idrac_ip}{IDRAC_API.IDRAC_DELL_MANAGERS}" \
+            f"{IDRAC_API.IDRAC_LLC}"
+
         response = self.api_get_call(r, headers)
         self.default_error_handler(response)
 
         data = response.json()
         self.api_endpoints = data
-        if 'Actions' in self.api_endpoints:
-            actions = self.api_endpoints["Actions"]
+        if IDRAC_JSON.Actions in self.api_endpoints:
+            actions = self.api_endpoints[IDRAC_JSON.Actions]
             action_keys = actions.keys()
             self.action_targets = [actions[k]['target'] for k in action_keys]
 
@@ -395,7 +456,7 @@ class IDracManager:
     def default_json_printer(json_data,
                              sort: Optional[bool] = True,
                              indents: Optional[int] = 4):
-        """json default stdout printer.
+        """default json stdout printer, it mainly used for debug.
         :param json_data:
         :param indents:
         :param sort:
@@ -425,10 +486,10 @@ class IDracManager:
         unfiltered_actions = {}
         full_redfish_names = {}
 
-        if 'Actions' not in json_data:
+        if IDRAC_JSON.Actions not in json_data:
             return unfiltered_actions, full_redfish_names
 
-        redfish_actions = json_data['Actions']
+        redfish_actions = json_data[IDRAC_JSON.Actions]
         for a in redfish_actions:
             _ca = redfish_actions[a]
             if a == "Oem" and isinstance(_ca, dict):
@@ -457,16 +518,16 @@ class IDracManager:
         :return:
         """
         action_dict = {}
-        if 'Members' not in json_data:
-            if 'Actions' in json_data:
+        if IDRAC_JSON.Members not in json_data:
+            if IDRAC_JSON.Actions in json_data:
                 return cls.discover_redfish_actions(cls, json_data)
             else:
                 return action_dict
 
-        member_data = json_data['Members']
+        member_data = json_data[IDRAC_JSON.Members]
         for m in member_data:
             if isinstance(m, dict):
-                if 'Actions' in m.keys():
+                if IDRAC_JSON.Actions in m.keys():
                     action = cls.discover_redfish_actions(cls, m)
                     action_dict.update(action)
 
@@ -501,22 +562,30 @@ class IDracManager:
 
         return action_dict
 
+    @cached_property
     def version_api(self, data_type: Optional[str] = "json") -> bool:
-        """Return true if a new version.
+        """Return true if IDRAC version 6.0 i.e. a new version.
         :return:
         """
         headers = {}
         if data_type == "json":
             headers.update(self.json_content_type)
-        r = f"https://{self.idrac_ip}/redfish/v1/Managers" \
+        r = f"{self._default_method}{self.idrac_ip}/redfish/v1/Managers" \
             f"/iDRAC.Embedded.1?$select=FirmwareVersion"
         response = self.api_get_call(r, headers)
         self.default_error_handler(response)
         data = response.json()
-        return int(data["FirmwareVersion"].replace(".", "")) >= 6000000
+        if 'FirmwareVersion' in data:
+            fw = data["FirmwareVersion"]
+            self.logger.info(f"IDRAC firmware {fw}")
+            return int(data["FirmwareVersion"].replace(".", "")) >= 6000000
+
+        return False
 
     @staticmethod
-    def filter_attribute(cls, json_data, attr_filter: Optional[str]):
+    def filter_attribute(cls,
+                         json_data,
+                         attr_filter: Optional[str]):
         """Filter attribute from json_data
         :param cls:
         :param json_data:
@@ -733,24 +802,26 @@ class IDracManager:
             )
 
     @staticmethod
-    def parse_error(error_response: requests.models.Response):
+    def parse_error(error_response: requests.models.Response) -> Tuple[str, str]:
         """Default Parser for error msg from
         JSON error response based on iDRAC.
         :param error_response:
         :return:
         """
         err_msg = "Default error."
+        messages = [""]
         err_resp = error_response.json()
         if 'error' in err_resp:
             err_data = err_resp['error']
             if 'message' in err_data:
                 err_msg = err_resp['error']['message']
             if '@Message.ExtendedInfo' in err_data:
+                messages = [m['Message'] for m in err_data['@Message.ExtendedInfo'] if 'Message' in m]
                 extended_err = err_data['@Message.ExtendedInfo'][-1]
                 err_msg = [f"{k}: {v}" for k, v in extended_err.items() if "@" not in k]
                 err_msg = "\n".join(err_msg)
 
-        return err_msg
+        return err_msg, " ".join(messages)
 
     @staticmethod
     def default_patch_success(cls,
@@ -772,11 +843,12 @@ class IDracManager:
                 or response.status_code == 202 or response.status_code == 204:
             return True
         else:
-            err_msg = IDracManager.parse_error(response)
-            raise PatchRequestFailed(
+            err_msg, json_error = IDracManager.parse_error(response)
+            e = PatchRequestFailed(
                 f"{err_msg}\nHTTP Status code: "
-                f"{response.status_code}"
+                f"{response.status_code}", json_error=json_error
             )
+            e.error_msg = json_error
 
     @staticmethod
     def default_post_success(cls,
@@ -800,10 +872,10 @@ class IDracManager:
                 or response.status_code == 204:
             return True
         else:
-            err_msg = IDracManager.parse_error(response)
+            err_msg, json_error = IDracManager.parse_error(response)
             raise PostRequestFailed(
                 f"{err_msg}\nHTTP Status code: "
-                f"{response.status_code}"
+                f"{response.status_code}", json_error=json_error
             )
 
     @staticmethod
@@ -826,10 +898,10 @@ class IDracManager:
                 or response.status_code == 204:
             return True
         else:
-            err_msg = IDracManager.parse_error(response)
+            err_msg, json_error = IDracManager.parse_error(response)
             raise DeleteRequestFailed(
                 f"{err_msg}\nHTTP Status code: "
-                f"{response.status_code}"
+                f"{response.status_code}", json_error=json_error
             )
 
     def api_async_del_until_complete(self, r, headers):
@@ -904,12 +976,13 @@ class IDracManager:
             headers.update(self.json_content_type)
 
         if do_expanded:
-            r = f"https://{self.idrac_ip}{resource}{self.expanded()}"
+            r = f"{self._default_method}{self.idrac_ip}{resource}{self.expanded()}"
         else:
-            r = f"https://{self.idrac_ip}{resource}"
+            r = f"{self._default_method}{self.idrac_ip}{resource}"
 
         if not do_async:
             response = self.api_get_call(r, headers)
+            self.query_counter += 1
             self.default_error_handler(response)
         else:
             loop = asyncio.get_event_loop()
@@ -924,7 +997,7 @@ class IDracManager:
             data = data[key]
 
         save_if_needed(filename, data)
-        return CommandResult(data, None, None)
+        return CommandResult(data, None, None, None)
 
     def base_patch(self,
                    resource: str,
@@ -949,13 +1022,19 @@ class IDracManager:
         else:
             pd = payload
 
+        self.logger.debug(f"Issuing patch request to "
+                          f"resource: {resource}, "
+                          f"payload: {json.dumps(pd)}")
+
         ok = False
+        err = None
         response = None
         try:
-            r = f"https://{self.idrac_ip}{resource}"
+            r = f"{self._default_method}{self.idrac_ip}{resource}"
             if not do_async:
                 if self._is_debug:
                     self.logger.debug(json.dumps(pd))
+
                 response = self.api_patch_call(
                     r, json.dumps(pd), headers
                 )
@@ -973,9 +1052,9 @@ class IDracManager:
             self.logger.critical(
                 pf, exc_info=self._is_debug
             )
-            pass
+            err = pf
 
-        return CommandResult(self.api_success_msg(ok), None, response)
+        return CommandResult(self.api_success_msg(ok), None, response, err)
 
     def base_post(self,
                   resource: str,
@@ -1003,15 +1082,16 @@ class IDracManager:
             pd = payload
 
         ok = False
+        err = None
         response = None
         try:
-            r = f"https://{self.idrac_ip}{resource}"
+            r = f"{self._default_method}{self.idrac_ip}{resource}"
             if not do_async:
                 response = self.api_post_call(r, json.dumps(pd), headers)
                 if verbose:
-                    print(f"received status code {response.status_code}")
-                    print(f"received status code {response.headers}")
-                    print(f"received {response.raw}")
+                    self.logger.debug(f"received status code {response.status_code}")
+                    self.logger.debug(f"received status code {response.headers}")
+                    self.logger.debug(f"received {response.raw}")
                 ok = self.default_post_success(
                     self, response, expected=expected_status
                 )
@@ -1023,44 +1103,73 @@ class IDracManager:
                     )
                 )
         except PostRequestFailed as pf:
-            self.logger.critical(
-                pf, exc_info=self._is_debug
-            )
+            err = pf
+            self.logger.critical(pf, exc_info=self._is_debug)
 
-        return CommandResult(self.api_success_msg(ok), None, response)
+        return CommandResult(self.api_success_msg(ok), None, response, err)
 
     @staticmethod
-    def api_success_msg(status: bool) -> Dict:
-        """Default api success respond.
-        :param status:
-        :return:
+    def api_success_msg(status: bool,
+                        message_key: Optional[str] = "message",
+                        message=None) -> Dict:
+        """A default api success respond.
+        :param status: a status true of false
+        :param message_key: key we need add extra
+        :param message: message information data
+        :return: a dict
         """
+        if message is not None:
+            return {
+                "Status": status,
+                message_key: message
+            }
+
         return {"Status": status}
 
+    @staticmethod
+    def api_is_change_msg(status: bool,
+                          message_key: Optional[str] = "message",
+                          message=None) -> Dict:
+        """A default api msg when no change applied.
+        :param status: a status true of false
+        :param message_key: key we need add extra
+        :param message: message information data
+        :return: a dict
+        """
+        if message is not None:
+            return {
+                "Changed": status,
+                message_key: message
+            }
+        return {"Changed": status}
+
     def reboot(self,
-               power_state_attr="PowerState",
-               default_reboot_type="ForceRestart") -> dict:
-        """Check if power state is on , reboots a host.
+               do_watch: Optional[bool] = False,
+               power_state_attr: Optional[str] = "PowerState",
+               default_reboot_type: Optional[str] = "ForceRestart") -> dict:
+        """Check if power state is on, reboots a host.
         :return: return a dict stora if operation succeed..
         """
         result_data = {}
-        cmd_result = self.sync_invoke(
+
+        # state of chassis
+        cmd_chassis = self.sync_invoke(
             ApiRequestType.ChassisQuery,
             "chassis_service_query",
             data_filter=power_state_attr
         )
 
-        if isinstance(cmd_result.data, dict) and 'PowerState' in cmd_result.data:
-            pd_state = cmd_result.data[power_state_attr]
+        if isinstance(cmd_chassis.data, dict) and 'PowerState' in cmd_chassis.data:
+            pd_state = cmd_chassis.data[power_state_attr]
             if pd_state == 'On':
-                cmd_result = self.sync_invoke(
+                cmd_reboot = self.sync_invoke(
                     ApiRequestType.RebootHost, "reboot",
-                    reset_type=default_reboot_type
+                    reset_type=default_reboot_type, do_watch=do_watch
                 )
-                if 'Status' in cmd_result.data:
+                if 'Status' in cmd_reboot.data:
                     result_data.update(
                         {
-                            "Reboot": cmd_result.data['Status']
+                            "Reboot": cmd_reboot.data['Status']
                         }
                     )
             else:
@@ -1074,40 +1183,117 @@ class IDracManager:
 
         return result_data
 
-    def idrac_firmware(self):
+    @cached_property
+    def idrac_firmware(self) -> str:
         """Shared method return idrac firmware
+        :return: str: firmware.
         """
-        return self.base_query("/redfish/v1/Managers/iDRAC.Embedded.1", key="FirmwareVersion")
+        api_return = self.base_query(self.idrac_members,
+                                     key=IDRAC_JSON.FirmwareVersion)
+        return api_return.data
 
-    def idrac_last_reset(self):
+    def idrac_last_reset(self) -> datetime:
         """Shared method return idrac last reset time"""
-        return self.base_query("/redfish/v1/Managers/iDRAC.Embedded.1", key="LastResetTime")
+        idrac_reset_time = None
+        api_return = self.base_query(self.idrac_members,
+                                     key=IDRAC_JSON.LastResetTime)
+        try:
+            idrac_reset_time = datetime.fromisoformat(api_return.data)
+        except ValueError as ve:
+            self.logger.error(ve)
+        return idrac_reset_time
 
-    def idrac_current_time(self):
-        """Shared method return idrac current time"""
-        return self.base_query("/redfish/v1/Managers/iDRAC.Embedded.1", key="DateTime")
+    def idrac_current_time(self) -> datetime:
+        """Shared method return idrac current time, if idrac return none ISO format
+        :return: datetime
+        """
+        idrac_time = None
+        api_return = self.base_query(self.idrac_members,
+                                     key=IDRAC_JSON.Datatime)
+        try:
+            idrac_time = datetime.fromisoformat(api_return.data)
+        except ValueError as ve:
+            self.logger.error(ve)
+        return idrac_time
+
+    @staticmethod
+    def local_time_iso():
+        """local time in iso format"""
+        current_date = datetime.now()
+        return current_date.isoformat()
 
     def idrac_time_offset(self):
         """Shared method return idrac current time"""
-        return self.base_query("/redfish/v1/Managers/iDRAC.Embedded.1", key="DateTimeLocalOffset")
+        api_resp = self.base_query(self.idrac_members,
+                                   key=IDRAC_JSON.DateTimeLocalOffset)
+        return api_resp.data
 
-    def idrac_manage_chassis(self):
-        """Shared method return idrac managed chassis list as json"""
-        links = self.base_query("/redfish/v1/Managers/iDRAC.Embedded.1", key="Links")
-        if links.data is not None and 'ManagerForChassis' in links.data:
-            return links.data['ManagerForChassis']
-        return links
+    @cached_property
+    def idrac_manage_chassis(self) -> str:
+        """Shared method return idrac managed chassis list as json
+        :return: str: manage chassis i.e. /redfish/v1/Chassis/System.Embedded.1
+        """
+        api_resp = self.base_query(self.idrac_members, key=IDRAC_JSON.Links)
+        if api_resp.data is not None and IDRAC_JSON.ManageChassis in api_resp.data:
+            if isinstance(api_resp.data, dict):
+                manage_chassis = api_resp.data[IDRAC_JSON.ManageChassis]
+                self._manage_chassis_obs = manage_chassis
+                return self.value_from_json_list(
+                    manage_chassis, IDRAC_JSON.Data_id
+                )
+        else:
+            self.logger.error("")
+        return ""
 
-    def idrac_manage_servers(self):
-        """Shared method return idrac managed servers list as json"""
-        links = self.base_query("/redfish/v1/Managers/iDRAC.Embedded.1", key="Links")
-        if links.data is not None and 'ManagerForServers' in links.data:
-            return links.data['ManagerForServers']
-        return links
+    @cached_property
+    def idrac_members(self) -> str:
+        """Shared method return idrac manage members servers list as json
+        /redfish/v1/Managers/iDRAC.Embedded.1
+        """
+        cmd_result = self.base_query(f"{IDRAC_API.IDRAC_MANAGER}", key=IDRAC_JSON.Members)
+        return self.value_from_json_list(cmd_result.data, IDRAC_JSON.Data_id)
 
+    @staticmethod
+    def value_from_json_list(json_obj, k):
+        """Parse json object dict.  If resp is json list [] get a key from last element
+        otherwise if a dict return value from a dict.
+        """
+        if isinstance(json_obj, list) and len(json_obj) > 0:
+            list_flat = json_obj[-1]
+            if isinstance(list_flat, dict):
+                if k in list_flat:
+                    return list_flat[k]
+        elif isinstance(json_obj, dict):
+            if k in json_obj:
+                return json_obj[k]
+        elif isinstance(json_obj, str):
+            return json_obj
+
+    @cached_property
+    def idrac_manage_servers(self) -> str:
+        """Shared method return idrac managed servers list as json
+        "/redfish/v1/Systems/System.Embedded.1"
+        """
+        api_resp = self.base_query(self.idrac_members, key=IDRAC_JSON.Links)
+        if api_resp.data is not None and IDRAC_JSON.ManagerServers in api_resp.data:
+            if isinstance(api_resp.data, dict):
+                manage_servers = api_resp.data[IDRAC_JSON.ManagerServers]
+                self._manage_servers_obs = manage_servers
+                return self.value_from_json_list(
+                    manage_servers, IDRAC_JSON.Data_id
+                )
+        else:
+            self.logger.error("")
+        return ""
+
+    @cached_property
     def idrac_id(self):
-        """Shared method return idrac current time"""
-        return self.base_query("/redfish/v1/Managers/iDRAC.Embedded.1", key="Id")
+        """Shared method return idrac id, System.Embedded.1"""
+        self.base_query(self.idrac_manage_servers, key=IDRAC_JSON.Id)
+        api_resp = self.base_query(self.idrac_manage_servers, key=IDRAC_JSON.Id)
+        if api_resp is None:
+            self.logger.critical(f"failed obtain {IDRAC_JSON.Id}")
+        return api_resp.data
 
     @staticmethod
     def base_parser(is_async: Optional[bool] = True,
@@ -1116,11 +1302,16 @@ class IDracManager:
                     is_remote_share: Optional[bool] = False,
                     is_reboot: Optional[bool] = False):
         """This idrac_ctl optional parser for all sub command.
-        Each sub-command can add additional optional flags
-        and args.
-        :return:
+        Each sub-command can add additional optional flags and args.
+        :return: argparse.ArgumentParser
         """
-        cmd_parser = argparse.ArgumentParser(add_help=False)
+        cmd_parser = argparse.ArgumentParser(
+            add_help=False, formatter_class=CustomArgumentDefaultsHelpFormatter
+        )
+
+        output_parser = cmd_parser.add_argument_group('output', 'Output related options')
+        chassis_parser = cmd_parser.add_argument_group('chassis', 'Chassis state options')
+
         if is_async:
             cmd_parser.add_argument(
                 '-a', '--async', action='store_true',
@@ -1130,21 +1321,21 @@ class IDracManager:
             )
 
         if is_expanded:
-            cmd_parser.add_argument(
+            output_parser.add_argument(
                 '-e', '--expanded', action='store_true',
                 required=False, dest="do_expanded",
                 default=False,
-                help="expanded request for deeper view."
+                help="expanded view, depend. it allows viewing more detail IDRAC data."
             )
         if is_file_save:
-            cmd_parser.add_argument(
+            output_parser.add_argument(
                 '-f', '--filename', required=False, default="",
                 type=str,
-                help="filename if we need to save a respond to a file."
+                help="filename, if we need to save a respond to a file."
             )
 
         if is_reboot:
-            cmd_parser.add_argument(
+            chassis_parser.add_argument(
                 '-r', '--reboot', action='store_true',
                 required=False, dest="do_reboot",
                 default=False,
@@ -1219,62 +1410,64 @@ class IDracManager:
         :raise UnexpectedResponse if header not present.
         """
         resp_hdr = response.headers
-        if 'Location' not in resp_hdr:
+        if IDRAC_JSON.Location not in resp_hdr:
             raise UnexpectedResponse(
                 "There is no location in the response header. "
                 "(not all api create job id)"
             )
-
-        location = response.headers['Location']
+        location = response.headers[IDRAC_JSON.Location]
         job_id = location.split("/")[-1]
         return job_id
 
     @staticmethod
-    def schedule_job(
+    def schedule_job_request(
             reboot_type: ScheduleJobType,
-            start_time: Optional[str],
+            start_time_isofmt: Optional[str],
             duration_time: Optional[int]) -> dict:
-        """Schedule a job.
-        :param reboot_type: reboot types.
-        :param start_time: start time for a job
+        """Create a JSON payload for schedule a job.
+
+        :param reboot_type: reboot type, ScheduleJobType
+        :param start_time_isofmt: start time for a job in ISO format.
         :param duration_time: duration for a job
         :return:
         """
         if reboot_type == ScheduleJobType.NoReboot:
             pd = {
-                "@Redfish.SettingsApplyTime": {
-                    "ApplyTime": "InMaintenanceWindowOnReset",
-                    "MaintenanceWindowStartTime": start_time,
-                    "MaintenanceWindowDurationInSeconds": duration_time
+                IDRAC_JSON.RedfishSettingsApplyTime: {
+                    IDRAC_JSON.ApplyTime: JobApplyTypes.InMaintenance,
+                    IDRAC_JSON.MaintenanceWindowStartTime: start_time_isofmt,
+                    IDRAC_JSON.MaintenanceWindowDuration: duration_time
                 }
             }
         elif reboot_type == ScheduleJobType.AutoReboot:
             pd = {
-                "@Redfish.SettingsApplyTime": {
-                    "ApplyTime": "AtMaintenanceWindowStart",
-                    "MaintenanceWindowStartTime": start_time,
-                    "MaintenanceWindowDurationInSeconds": duration_time
+                IDRAC_JSON.RedfishSettingsApplyTime: {
+                    IDRAC_JSON.ApplyTime: JobApplyTypes.AtMaintenance,
+                    IDRAC_JSON.MaintenanceWindowStartTime: start_time_isofmt,
+                    IDRAC_JSON.MaintenanceWindowDuration: duration_time
                 }
             }
         elif reboot_type == ScheduleJobType.OnReset:
             pd = {
-                "@Redfish.SettingsApplyTime": {
-                    "ApplyTime": "OnReset"
+                IDRAC_JSON.RedfishSettingsApplyTime: {
+                    IDRAC_JSON.ApplyTime: JobApplyTypes.OnReset
                 }
             }
         elif reboot_type == ScheduleJobType.Immediate:
             pd = {
-                "@Redfish.SettingsApplyTime": {
-                    "ApplyTime": "Immediate"
+                IDRAC_JSON.RedfishSettingsApplyTime: {
+                    IDRAC_JSON.ApplyTime: JobApplyTypes.Immediate
                 }
             }
         else:
-            raise ValueError("Invalid settings apply time.")
+            raise InvalidArgumentFormat(
+                "Invalid settings apply time.")
         return pd
 
     def parse_task_id(self, data):
-        """Parse input data and try to get job id from the header or response.
-        :param data:
+        """Parses input data and try to get a job id from the header
+        or http response.
+        :param data:  http response or CommandResult
         :return:
         """
         # get response from extra
@@ -1314,3 +1507,84 @@ class IDracManager:
             return data
 
         return {}
+
+    @staticmethod
+    def make_future_job_ts(start_date: str,
+                           start_time: str,
+                           is_json_string=False) -> str:
+        """Make a future time for a maintenance task.
+        It takes start_date in format YYYY-MM-DD
+        and starts time in format HH:MM:SS and return
+        JSON string time in ISO format.
+
+        "2023-02-08T01:01:01.000001"
+
+       :param start_date: start date for a job YYYY-MM-DD
+       :param start_time: a start time for a job HH:MM:SS
+       :param is_json_string return python string or JSON string.
+       :raise: MissingMandatoryArguments if mandatory args missing.
+       :raise: InvalidArgumentFormat if format of the input is invalid.
+       """
+        if start_date is None or len(start_date) == 0:
+            raise MissingMandatoryArguments(
+                "A maintenance job requires a start date.")
+
+        if start_time is None or len(start_time) == 0:
+            raise MissingMandatoryArguments(
+                "A maintenance job requires a start time.")
+        try:
+            local_timestamp = datetime.now()
+            ts = f'{start_date}T{start_time}.000001'
+            start_timestamp = datetime.fromisoformat(ts)
+            if start_timestamp < local_timestamp:
+                raise InvalidArgumentFormat(
+                    f"Start time is in the past local time "
+                    f"{str(local_timestamp)} {str(start_timestamp)}")
+
+            # note we always parse to make sure we can convert.
+            json_str = json.dumps(start_timestamp.isoformat())
+            if is_json_string:
+                return json_str
+            else:
+                return start_timestamp.isoformat()
+        except ValueError as ve:
+            raise InvalidArgumentFormat(str(ve))
+        except TypeError as te:
+            raise InvalidArgumentFormat(str(te))
+
+    def create_apply_time_req(self,
+                              apply: str,
+                              start_date: str,
+                              start_time: str,
+                              default_duration):
+        """
+        :param start_date
+        :param start_time
+        :param default_duration
+        :param apply time auto-boot, maintenance, on-reset
+        :raise InvalidArgumentFormat
+        """
+        if apply.strip().lower() == "auto-boot":
+            start_timestamp = self.make_future_job_ts(start_date, start_time)
+            return self.schedule_job_request(
+                ScheduleJobType.AutoReboot,
+                start_time_isofmt=start_timestamp,
+                duration_time=default_duration
+            )
+        elif apply.strip().lower() == "maintenance":
+            start_timestamp = self.make_future_job_ts(start_date, start_time)
+            return self.schedule_job_request(
+                ScheduleJobType.NoReboot,
+                start_time_isofmt=start_timestamp,
+                duration_time=default_duration
+            )
+        elif apply.strip().lower() == "on-reset":
+            return self.schedule_job_request(
+                ScheduleJobType.OnReset,
+                start_time_isofmt=None,
+                duration_time=None
+            )
+        else:
+            ValueError("Unknown apply time")
+
+        return {}
